@@ -113,13 +113,12 @@ def test_association(
 
     adata = adata.copy() if copy else adata
 
-    if "pseudotime_list" not in adata.uns:
+    if "t" not in adata.obs:
         raise ValueError(
             "You need to run `tl.pseudotime` before testing for association."
+            + "Or add a precomputed pseudotime at adata.obs['t'] for single segment."
         )
 
-    graph = adata.uns["graph"]
-
     if reapply_filters & ("stat_assoc_list" in adata.uns):
         stat_assoc_l = list(adata.uns["stat_assoc_list"].values())
         # stat_assoc_l = list(map(lambda x: pd.DataFrame(x,index=x["features"]),stat_assoc_l))
@@ -191,6 +190,154 @@ def test_association(
     return adata if copy else None
 
 
+def test_association_monocle3(
+    adata: AnnData,
+    qval_cut: float = 1e-4,
+    n_jobs: bool = 1,
+    copy: bool = False,
+    **kwargs,
+):
+    """\
+    Determine a set of genes significantly associated with the trajectory.
+
+    the function `graph_test` *monocle3* R package is called,
+    using `neighbor_graph = 'principal_graph'` parameter. The statistic tells you
+    whether cells at nearby positions on a trajectory will have similar (or dissimilar)
+    expression levels for the gene being tested.
+
+
+    Parameters
+    ----------
+    adata
+        Annotated data matrix.
+    qval_cut
+        cutoff on the corrected p-values to consider a feature as significant.
+    n_jobs
+        number of cpu processes used to perform the test.
+    spline_df
+        dimension of the basis used to represent the smooth term.
+    copy
+        Return a copy instead of writing to adata.
+    Returns
+    -------
+    adata : anndata.AnnData
+        if `copy=True` it returns or else add fields to `adata`:
+        `.var['status']`
+            The feature could be tested.
+        `.var['p_value']`
+            p-values from statistical test.
+        `.var['morans_test_statistic']`
+            Statistics of the Moran test
+        `.var['morans_I']`
+            Moran’s I is a measure of multi-directional and multi-dimensional spatial autocorrelation.
+        `.var['q_value']`
+            corrected values from multiple testing.
+        '.var['signi']`
+            feature has q_value below cutoff.
+
+    """
+
+    Rpy2, R, rstats, monocle3, Formula = importeR(
+        "testing feature association to the tree", "monocle3"
+    )
+    check = [type(imp) == str for imp in [Rpy2, R, rstats, monocle3, Formula]]
+    if any(check):
+        idx = np.argwhere(
+            [type(imp) == str for imp in [Rpy2, R, rstats, monocle3, Formula]]
+        ).min()
+        raise Exception(np.array([Rpy2, R, rstats, monocle3, Formula])[idx])
+
+    logg.info(
+        "test features for association with the trajectory, monocle3 way",
+        reset=True,
+        end="\n",
+    )
+
+    adata = adata.copy() if copy else adata
+
+    import scipy.sparse as sp
+    from rpy2.robjects.packages import importr
+    import rpy2.robjects as ro
+    from rpy2.robjects.conversion import py2rpy
+    from scFates.tools.graph_fitting import get_data
+    import anndata2ri
+
+    anndata2ri.activate()
+
+    from . import __path__
+
+    Rfun = os.path.join(__path__[0], "_test_monocle3.R")
+    rsource = ro.r["source"]
+    rsource(Rfun)
+    _test_monocle3 = ro.globalenv["test_monocle3"]
+
+    # prepare data
+    B = sp.csc_matrix(adata.uns["graph"]["B"]).astype(float)
+    F = pd.DataFrame(adata.uns["graph"]["F"])
+    F.columns = ["Y_" + str(i) for i in np.arange(adata.obsm["X_R"].shape[1]) + 1]
+    pr_graph_cell_proj_closest_vertex = pd.DataFrame(
+        {"1": np.argmax(adata.obsm["X_R"], axis=1) + 1}, index=adata.obs_names
+    )
+    cells = adata.obs_names
+    genes = adata.var_names
+    X = sp.csc_matrix(adata.X)
+    UMAP, use_rep = get_data(
+        adata, adata.uns["graph"]["use_rep"], adata.uns["graph"]["ndims_rep"]
+    )
+    UMAP = UMAP.values
+
+    # prevent double log transformation
+    if ("log1p" in adata.uns) & ("expression_family" not in kwargs):
+        kwargs["expression_family"] = "uninormal"
+
+    # Avoiding rBind error for large datasets
+    # see: https://github.com/cole-trapnell-lab/monocle3/issues/509
+    if ("neighbor_graph" not in kwargs) & (adata.shape[0] <= 10000):
+        kwargs["neighbor_graph"] = "principal_graph"
+    elif ("neighbor_graph" not in kwargs) & (adata.shape[0] > 10000):
+        kwargs["neighbor_graph"] = "knn"
+
+    logg.info(
+        f"    neighbor_graph set to {kwargs['neighbor_graph']}",
+        end=" " if settings.verbosity > 2 else "\n",
+    )
+
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        pr_graph_test_res = _test_monocle3(
+            X,
+            genes,
+            cells,
+            UMAP,
+            pr_graph_cell_proj_closest_vertex,
+            F,
+            B,
+            n_jobs,
+            **kwargs,
+        )
+    for c in pr_graph_test_res.columns:
+        adata.var[c] = pr_graph_test_res[c]
+
+    adata.var["signi"] = pr_graph_test_res.q_value < qval_cut
+    logg.info(
+        "    found " + str(sum(adata.var["signi"])) + " significant features",
+        time=True,
+        end=" " if settings.verbosity > 2 else "\n",
+    )
+
+    logg.hint(
+        "added\n"
+        "    .var['status'] the feature could be tested.\n"
+        "    .var['p_value'] p-values from statistical test.\n"
+        "    .var['morans_test_statistic'] Statistics of the Moran test.\n"
+        "    .var['morans_I'] Moran’s I is a measure of multi-directional and multi-dimensional spatial autocorrelation.\n"
+        "    .var['q_value'] corrected values from multiple testing.\n"
+        "    .var['signi'] feature has q_value below cutoff."
+    )
+
+    return adata if copy else None
+
+
 def test_assoc(data, spline_df):
     sdf = data[0]
     sdf["exp"] = data[1]
@@ -262,11 +409,8 @@ def apply_filters(adata, stat_assoc_l, fdr_cut, A_cut, st_cut, prefix=""):
 
     # saving results
     stat_assoc[prefix + "signi"] = stat_assoc[prefix + "st"] > st_cut
-
-    if set(stat_assoc.columns.tolist()).issubset(adata.var.columns):
-        adata.var[stat_assoc.columns] = stat_assoc
-    else:
-        adata.var = pd.concat([adata.var, stat_assoc], axis=1)
+    for c in stat_assoc.columns:
+        adata.var[c] = stat_assoc[c]
 
     names = np.arange(len(stat_assoc_l)).astype(str).tolist()
 
